package cn.com.qmth.scancentral.controller.admin;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import com.qmth.boot.tools.iterator.PageListIterator;
import com.qmth.boot.tools.iterator.SingletonIterator;

import cn.com.qmth.scancentral.bean.AbsentQueryDomain;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.entity.SubjectEntity;
import cn.com.qmth.scancentral.enums.ConditionType;
import cn.com.qmth.scancentral.enums.GroupType;
import cn.com.qmth.scancentral.model.ManualAbsentImportDTO;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.service.SubjectService;
import cn.com.qmth.scancentral.vo.AbsentInfoVo;
import cn.com.qmth.scancentral.vo.AbsentManualImportVo;
import cn.com.qmth.scancentral.vo.AbsentQueryVo;
import cn.com.qmth.scancentral.vo.OmrConditionsVo;
import cn.com.qmth.scancentral.vo.UpdateTimeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "缺考接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/check")
@Aac(strict = false, auth = true)
public class CheckController extends BaseController {

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @ApiOperation(value = "所有可用的识别对照条件")
    @PostMapping("omr/conditions")
    public List<OmrConditionsVo> omrConditions() {
        return Arrays.stream(ConditionType.values()).map(OmrConditionsVo::new).collect(Collectors.toList());
    }

    @ApiOperation(value = "缺考数据汇总")
    @PostMapping("absent/info")
    public AbsentInfoVo absentInfo(@RequestParam Long examId, @RequestParam(required = false) GroupType groupType,
            @RequestParam(required = false) String groupName) {
        return studentService.absentInfo(examId, groupType, groupName);
    }

    @ApiOperation(value = "查询缺考数据")
    @RequestMapping(value = "absent/query", method = RequestMethod.POST)
    public PageResult<AbsentQueryVo> absentQuery(@Validated AbsentQueryDomain query) {
        return studentService.absentQuery(query);
    }

    @ApiOperation(value = "查询缺考数据标识")
    @RequestMapping(value = "absent/summary", method = RequestMethod.POST)
    public List<String> absentSummary(@Validated AbsentQueryDomain query) {
        return studentService.absentSummary(query);
    }

    @ApiOperation(value = "导出缺考数据")
    @RequestMapping(value = "absent/export", method = RequestMethod.POST)
    public void absentExport(@Validated AbsentQueryDomain query, HttpServletResponse response) throws IOException {
        String fileName = URLEncoder.encode("缺考数据", "UTF-8");
        response.setHeader("Content-Disposition", "inline; filename=" + fileName + ".xlsx");
        response.setContentType("application/vnd.ms-excel");

        Map<String, SubjectEntity> subjectMap = new HashMap<>();
        PageListIterator<DataMap> iterator = new PageListIterator<DataMap>(100) {

            @Override
            public Collection<DataMap> getPageList(int pageNumber, int pageSize) {
                query.setPageNumber(pageNumber);
                query.setPageSize(pageSize);
                List<AbsentQueryVo> list = studentService.absentExportList(query);
                List<DataMap> ret = new ArrayList<>();
                if (CollectionUtils.isNotEmpty(list)) {
                    for (AbsentQueryVo vo : list) {
                        DataMap map = new DataMap();
                        if (GroupType.SUBJECT.equals(query.getGroupType())) {
                            map.put("科目代码", vo.getGroupName());
                            map.put("科目名称", getSubjectName(subjectMap, vo.getGroupName(), query.getExamId()));
                        } else {
                            map.put(query.getGroupType().getName(), vo.getGroupName());
                        }
                        map.put("考生总数", getStringVal(vo.getTotalCount()));
                        map.put("已扫描", getStringVal(vo.getScannedCount()));
                        map.put("未扫描", getStringVal(vo.getUnexistCount()));
                        map.put("指定缺考", getStringVal(vo.getManualAbsentCount()));
                        map.put("识别缺考", getStringVal(vo.getOmrAbsentCount()));
                        map.put("缺考异常", getStringVal(vo.getAbsentSuspectCount()));
                        ret.add(map);
                    }
                }
                return ret;
            }
        };
        String[] head;
        if (GroupType.SUBJECT.equals(query.getGroupType())) {
            head = new String[] { "科目代码", "科目名称", "考生总数", "已扫描", "未扫描", "指定缺考", "识别缺考", "缺考异常" };
        } else {
            head = new String[] { query.getGroupType().getName(), "考生总数", "已扫描", "未扫描", "指定缺考", "识别缺考", "缺考异常" };
        }
        ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
        writer.writeDataMaps("缺考核对", null, head, iterator);
        writer.output(response.getOutputStream());
    }

    private String getSubjectName(Map<String, SubjectEntity> subMap, String subjectCode, Long examId) {
        SubjectEntity subject = subMap.get(subjectCode);
        if (subMap.get(subjectCode) == null) {
            subject = subjectService.findByExamIdAndCode(examId, subjectCode);
            if (subject == null) {
                throw new StatusException("未找到科目信息：" + subjectCode);
            }
            subMap.put(subjectCode, subject);
        }
        return subject.getName();
    }

    private String getStringVal(Integer val) {
        if (val == null) {
            return "0";
        }
        return val.toString();
    }

    @PostMapping("absent/manual/import")
    @ApiOperation(value = "导入指定缺考名单")
    public AbsentManualImportVo absentManualImport(@RequestParam Long examId, @RequestParam MultipartFile file) {
        return studentService.absentManualImport(examId, file);
    }

    @ApiOperation(value = "下载导入指定缺考名单模板")
    @PostMapping("absent/manual/template")
    public void getImportTemplate() throws IOException {
        ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
        writer.writeObjects("指定缺考名单", null, ManualAbsentImportDTO.class,
                new SingletonIterator<>(new ManualAbsentImportDTO()));
        exportFile("指定缺考名单导入模板.xlsx", writer);
    }

    @ApiOperation(value = "更新为指定缺考")
    @PostMapping("absent/manual/update")
    public UpdateTimeVo absentManualUpdate(@RequestParam Long examId, @RequestParam String examNumber) {
        return studentService.absentManualUpdate(examId, examNumber);
    }

    @ApiOperation(value = "取消缺考标记嫌疑")
    @PostMapping("absent/suspect/remove")
    public UpdateTimeVo absentSuspectRemove(@RequestParam Long examId, @RequestParam String examNumber) {
        return studentService.absentSuspectUpdate(examId, examNumber, false);
    }
}