package cn.com.qmth.scancentral.controller.scan;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.bean.answertask.ReleaseVo;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskSaveDomain;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskSaveVo;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskStatusVo;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskVo;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.entity.StudentAnswerTaskEntity;
import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.enums.StudentAnswerTaskStatus;
import cn.com.qmth.scancentral.service.StudentAnswerTaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "扫描功能-数据抽查接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/scan/task/student/answer")
@Aac(strict = false, auth = true)
public class StudentAnswerTaskController extends BaseController {

    @Autowired
    private StudentAnswerTaskService studentAnswerTaskService;

    @Autowired
    private ConcurrentService concurrentService;

    @ApiOperation(value = "任务获取")
    @PostMapping("get")
    public StudentAnswerTaskVo get(@RequestParam Long examId) {
        User user = getAccessUser();
        return studentAnswerTaskService.getTask(examId, user.getAccount());
    }

    @ApiOperation(value = "抽查任务提交")
    @PostMapping("save")
    public StudentAnswerTaskSaveVo save(@RequestBody StudentAnswerTaskSaveDomain domain) {
        User user = getAccessUser();
        if (domain.getId() == null) {
            throw new ParameterException("id不能为空");
        }
        if (domain.getTag() == null) {
            throw new ParameterException("tag不能为空");
        }
        StudentAnswerTaskEntity task = studentAnswerTaskService.getById(domain.getId());
        if (task == null) {
            throw new ParameterException("任务不存在");
        }
        // if (StudentAnswerTaskStatus.PROCESSED.equals(task.getStatus())) {
        // throw new ParameterException("任务已处理");
        // }
        if (StudentAnswerTaskStatus.WAITING.equals(task.getStatus())
                && !studentAnswerTaskService.hasApplied(task, user.getAccount())) {
            throw new ParameterException("任务非本人领取");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT_ANSWER_GROUP + "-" + task.getGroupId()).readLock().lock();
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + task.getStudentId()).writeLock().lock();
        try {
            return studentAnswerTaskService.submitTask(domain, user);
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + task.getStudentId()).writeLock().unlock();
            concurrentService.getReadWriteLock(LockType.STUDENT_ANSWER_GROUP + "-" + task.getGroupId()).readLock()
                    .unlock();
        }
    }

    @ApiOperation(value = "抽查任务状态")
    @PostMapping("status")
    public StudentAnswerTaskStatusVo getStatus(@RequestParam Long examId) {
        User user = getAccessUser();
        return studentAnswerTaskService.getStatus(examId, user);
    }

    @ApiOperation(value = "抽查任务释放")
    @PostMapping("release")
    public ReleaseVo release(@RequestParam Long examId) {
        User user = getAccessUser();
        ReleaseVo vo = new ReleaseVo();
        studentAnswerTaskService.releaseByUser(examId, user.getAccount());
        vo.setSuccess(true);
        return vo;
    }

    @ApiOperation(value = "抽查任务历史")
    @RequestMapping(value = "history", method = RequestMethod.POST)
    public StudentAnswerTaskVo history(@RequestParam Long examId, @RequestParam(required = false) Long id,@RequestParam(required = false) Boolean next) {
        User user = getAccessUser();
        return studentAnswerTaskService.history(examId, id, user.getAccount(),next);
    }
}
