package cn.com.qmth.scancentral.service.impl;

import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancentral.bean.answergroup.StudentAnswerGroupCondition;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupCountVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupIdVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupMarkedVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupSummaryQuery;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupSummaryVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupVo;
import cn.com.qmth.scancentral.dao.StudentAnswerGroupDao;
import cn.com.qmth.scancentral.entity.StudentAnswerGroupEntity;
import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.enums.StudentAnswerTaskStatus;
import cn.com.qmth.scancentral.exception.NotFoundExceptions;
import cn.com.qmth.scancentral.service.PaperService;
import cn.com.qmth.scancentral.service.StudentAnswerGroupService;
import cn.com.qmth.scancentral.service.StudentAnswerTaskService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.util.BatchSetDataUtil;

@Service
public class StudentAnswerGroupServiceImpl extends ServiceImpl<StudentAnswerGroupDao, StudentAnswerGroupEntity>
        implements StudentAnswerGroupService {
    @Autowired
    private ConcurrentService concurrentService;
    @Autowired
    private StudentAnswerTaskService taskService;
    
    @Autowired
    private PaperService paperService;
    
    @Autowired
    private StudentService studentService;

    @Transactional
    @Override
    public void deleteById(Long id) {
        taskService.deleteByGroupId(id);
        this.removeById(id);
    }

    @Transactional
    @Override
    public void resetById(Long id) {
        StudentAnswerGroupEntity group = this.getById(id);
        if (group == null) {
            throw new ParameterException("分组不存在");
        }
        taskService.resetByGroup(group);
    }

    @Transactional
    @Override
    public void buildTaskById(Long groupId) {
        StudentAnswerGroupEntity group = this.getById(groupId);
        List<Long> studentIds = studentService.findStudentAnswerTask(group.getConditions());
        if(CollectionUtils.isEmpty(studentIds)) {
            return; 
        }
        new BatchSetDataUtil<Long>() {
            @Override
            protected void setData(List<Long> dataList) {
                taskService.saveTask(group, dataList);
            }
        }.setDataForBatch(studentIds, 1000);
        
    }

    @Override
    public StudnetAnswerGroupVo listByExamId(Long examId) {
        StudnetAnswerGroupVo vo = new StudnetAnswerGroupVo();
        QueryWrapper<StudentAnswerGroupEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerGroupEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerGroupEntity::getExamId, examId);
        wrapper.last("LIMIT 1");
        StudentAnswerGroupEntity group = this.baseMapper.selectOne(wrapper);
        if(group==null) {
            throw NotFoundExceptions.NO_STUDENT_ANSWER_GROUP;
        }
        vo.setId(group.getId());
        vo.setBuilding(!concurrentService.getSemaphore(LockType.STUDENT_ANSWER_GROUP_BUILD + "-" + group.getId()).isAvailable());
        vo.setDeleting(!concurrentService.getSemaphore(LockType.STUDENT_ANSWER_GROUP_DELETE + "-" + group.getId()).isAvailable());
        vo.setReseting(!concurrentService.getSemaphore(LockType.STUDENT_ANSWER_GROUP_RESET + "-" + group.getId()).isAvailable());
        vo.setConditions(group.getConditions());
        vo.setTotalCount(group.getTotalCount());
        vo.setTodoCount(
                taskService.getCountByExamAndGroupAndStatus(examId, group.getId(), StudentAnswerTaskStatus.WAITING));
        return vo;
    }


    private List<StudentAnswerGroupEntity> findByExamId(Long examId) {
        QueryWrapper<StudentAnswerGroupEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerGroupEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerGroupEntity::getExamId, examId);
        return baseMapper.selectList(wrapper);
    }

    @Transactional
    @Override
    public void updateTotalCount(Long id) {
        if (id == null) {
            throw new ParameterException("Id不能为空");
        }
        baseMapper.updateTotalCount(id);
    }


    @Transactional
    @Override
    public void updateTotalCountByExamId(Long examId) {
        List<StudentAnswerGroupEntity> list = this.findByExamId(examId);
        for (StudentAnswerGroupEntity groupEntity : list) {
            this.updateTotalCount(groupEntity.getId());
        }
    }

    @Override
    public StudnetAnswerGroupCountVo countByCondition(StudentAnswerGroupCondition domain) {
        if (domain.getExamId() == null) {
            throw new ParameterException("examId不能为空");
        }
        StudnetAnswerGroupCountVo vo = new StudnetAnswerGroupCountVo();
        if(domain.getAssigned() ==null ||domain.getAssigned()==false ) {
            domain.setStartTime(null);
            domain.setEndTime(null);
        }
        vo.setTotalCount(studentService.queryStudentAnswerGroupCount(domain));
        return vo;
    }

    @Transactional
    @Override
    public StudnetAnswerGroupIdVo saveByCondition(StudentAnswerGroupCondition domain) {
        if (domain.getExamId() == null) {
            throw new ParameterException("examId不能为空");
        }
        QueryWrapper<StudentAnswerGroupEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerGroupEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerGroupEntity::getExamId, domain.getExamId());
        wrapper.last("LIMIT 1");
        StudentAnswerGroupEntity oldgroup = this.baseMapper.selectOne(wrapper);
        if (oldgroup != null) {
            throw new ParameterException("已存在任务组");
        }
        StudentAnswerGroupEntity group = new StudentAnswerGroupEntity();
        group.setExamId(domain.getExamId());
        group.setTotalCount(0);
        if(domain.getAssigned() ==null ||domain.getAssigned()==false ) {
            domain.setStartTime(null);
            domain.setEndTime(null);
        }
        group.setConditions(domain);
        this.save(group);
        StudnetAnswerGroupIdVo vo=new StudnetAnswerGroupIdVo();
        vo.setId(group.getId());
        return vo;
    }

    @Override
    public StudnetAnswerGroupSummaryVo summary(StudnetAnswerGroupSummaryQuery query) {
        StudnetAnswerGroupSummaryVo vo=new StudnetAnswerGroupSummaryVo();
        vo.setSystem(paperService.getMismatchCountByExam(query.getExamId()));
        vo.setMark(studentService.getStudentAnswerCount(query)); 
        return vo;
    }

    @Override
    public List<StudnetAnswerGroupMarkedVo> marked(Long examId) {
        return studentService.getStudentAnswerMarkedList(examId);
    }
}