package cn.com.qmth.scancentral.task.thread;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.qmth.boot.core.concurrent.model.Semaphore;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.service.StudentAnswerGroupService;

public class StudentAnswerGroupBuildThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(StudentAnswerGroupBuildThread.class);

    private Long groupId;

    private StudentAnswerGroupService groupService;

    private ConcurrentService concurrentService;

    private Semaphore semaphore;

    public StudentAnswerGroupBuildThread(Long groupId, StudentAnswerGroupService groupService, ConcurrentService concurrentService,
            Semaphore semaphore) {
        this.groupId = groupId;
        this.groupService = groupService;
        this.concurrentService = concurrentService;
        this.semaphore = semaphore;
    }

    @Override
    public void run() {
        try {
            log.info("StudentAnswerGroup[" + groupId + "] build task start");
            concurrentService.getReadWriteLock(LockType.STUDENT_ANSWER_GROUP + "-" + groupId).readLock().lock();
            groupService.buildTaskById(groupId);
            log.info("StudentAnswerGroup[" + groupId + "] build task finish");
        } catch (Exception e) {
            log.error("StudentAnswerGroup[" + groupId + "] build task error", e);
            throw new StatusException("抽查分组生成任务失败");
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT_ANSWER_GROUP + "-" + groupId).readLock().unlock();
            semaphore.release();
        }
    }

}
